﻿--
-- Very simple addon that adds a note to NPC tooltips when available.
-- If you want to add your own notes in-game, simply do
--  /script MobNotesDB["Name of mob"] = "Note"
-- (note that %t doesn't work with /script)
--
-- Database upgrades (i.e. if the addon is updated on the SVN with new notes)
-- will NOT overwrite or change any custom notes you have added.
--

--[[
<Pkekyo|> much easier to ninja out of combatlog what CAN'T be done imo - for the end user, atleast
<vhaarr> I'm not interested in your opinion
<ag`> :p
<kergoth> write a little addon to ninja those messages and inject them into mobnotes's SV, perhaps.  then it's still flexible for adding any notes, but would gain those bits automatically?
<vhaarr> kergoth: that could be interesting, at least if those "Your Polymorph fails. X is immune." and "Your Wound Poison fails. X is immune." etc messages are generated from the same event.
<vhaarr> kergoth: actually it's CHAT_MSG_SPELL_SELF_DAMAGE
<vhaarr> which is fired pretty often

-- Could still be interesting to fool around with it as a separate addon, but I
-- don't think I want it included here. Perhaps wait for 2.4 and see then.
--
--]]

-- Obviously bump this number if you add any new notes. Note that changing old
-- notes will not upgrade the users DB.
local DB_VERSION = 14

-- The default database should only include strategy-neutral comments, like what
-- mobs are capable of doing and how they can be crowd controlled, or if they
-- are immune to everything, like sheeps or stuns. Don't include any strategy
-- like "kill this mob first" or "have your offtank on this one".
local defaultDatabase = nil

local l = GetLocale()

local MNLDB = nil
local configDb = nil

local POPUP_TEXT = "Set a note for |cffff4488%s|r."
local TARGET_ERROR = "You must target something to add a note."
local NOTE_SET = "Note for |cffff4488%s|r set to \"|cff888888%s|r\"."
local SET = "Set"
local CANCEL = "Cancel"
local NOTE_PREFIX = "|cffff4488Note:|r "
-- Keybinding to output the note to chat
BINDING_HEADER_MOBNOTES = "MobNotes"
BINDING_NAME_MOBNOTES_PRINT = "Print Note"

if l == "deDE" then
	POPUP_TEXT = "Festlegen der Notiz für |cffff4488%s|r."
	TARGET_ERROR = "Du must ein Ziel auswählen um eine Notiz zu erstellen."
	NOTE_SET = "Notiz für |cffff4488%s|r festgelegt auf \"|cff888888%s|r\"."
	SET = "Festlegen"
	CANCEL = "Abbrechen"
	NOTE_PREFIX = "|cffff4488Notiz:|r "
elseif l == "zhCN" then
	POPUP_TEXT = "为|cffff4488%s|r设置注释."
	TARGET_ERROR = "你必须先选择一个目标来添加注释"
	NOTE_SET = "|cffff4488%s|r的注释更改为\"|cff888888%s|r\"."
	SET = "设置"
	CANCEL = "取消"
	NOTE_PREFIX = "|cffff4488注释：|r "
elseif l == "zhTW" then
	POPUP_TEXT = "為|cffff4488%s|r設定註解"
	TARGET_ERROR = "你必須先選擇一個目標來增加註解"
	NOTE_SET = "|cffff4488%s|r的註解更改為\"|cff888888%s|r\"."
	SET = "設定"
	CANCEL = "取消"
	NOTE_PREFIX = "|cffff4488註解: |r "
end

if l == "enUS" then
defaultDatabase = {
	-- The following are examples of how to populate the default DB.
	-- Please read the instructions at the top. NO STRATEGY INFORMATION IN DEFAULT DATABASE TIPS, PLEASE!
	[51346] = "Sometimes fly around carrying Horde banners. Aren't they neat?" -- Orgrimmar Grunt (4.0.3+)
	[68]    = "Watch out, or they'll Shoot you with their bow! I wish I was a Knight of Camelot." -- Stormwind City Guard
	[51371] = "Aww, they're so cute :D They can dismount you, though! Better tell Santa to stay away from this place." -- Darnassus Sentinel
	[3084]  = "Moo."
	[5595]  = "Oh noes, a slightly bigger version of Gimli! Better watch out or he'll drink all your mead." -- Ironforge Guard
}
elseif l == "deDE" then
elseif l == "zhCN" then
elseif l == "zhTW" then
end

-----------------------------------------------------------------------
-- Local utility
--

local function getNumericUniqueID(unitId)
	if not UnitExists(unitId) then return end
	local guid = UnitGUID(unitId)
	-- x % 8 has the same effect as x & 0x7 on numbers <= 0xf
	local masked = tonumber(guid:sub(5, 5), 16) % 8
	if masked == 3 then -- Mob
		return tonumber(guid:sub(7, 10), 16)
	elseif masked == 0 then -- Player
		return tonumber(guid:sub(13, 18), 16)
	end
end

local function addNoteFromPopup(note)
	if type(note) ~= "string" or note:trim():len() == 0 then note = nil end
	local t = UnitExists("target") and UnitName("target") or nil
	if t then
		local id = getNumericUniqueID("target")
		_G.MobNotesDB[id] = note
		print(NOTE_SET:format(t, tostring(note)))
		if MNLDB and configDb.ldbNoteOnText then
			MNLDB.text = note
		end
	end
end

local function addNote(input)
	if not UnitExists("target") then
		print(TARGET_ERROR)
		return
	end
	if type(input) ~= "string" or input:trim():len() == 0 then
		if not StaticPopupDialogs["MobNotes"] then
			StaticPopupDialogs["MobNotes"] = {
				text = nil,
				button1 = SET,
				button2 = CANCEL,
				whileDead = 1,
				hideOnEscape = 1,
				timeout = 0,
				OnShow = function(self)
					-- We have to do this onshow to reset the previous text
					local id = UnitExists("target") and getNumericUniqueID("target")
					self.editBox:SetText(_G.MobNotesDB[id] or "")
				end,
				OnHide = function(self)
					self.editBox:SetText("")
				end,
				EditBoxOnEnterPressed = function(self)
					addNoteFromPopup(self:GetText())
					self:GetParent():Hide()
				end,
				EditBoxOnEscapePressed = function(self)
					self:GetParent():Hide()
				end,
				OnAccept = function(self)
					addNoteFromPopup(self.editBox:GetText())
				end,
				hasEditBox = 1,
				preferredIndex = STATICPOPUP_NUMDIALOGS,
			}
		end
		StaticPopupDialogs["MobNotes"].text = POPUP_TEXT:format(UnitName("target"))
		StaticPopup_Show("MobNotes")
	else
		local t = UnitName("target")
		local id = getNumericUniqueID("target")
		_G.MobNotesDB[id] = input
		print(NOTE_SET:format(t, input))
		if MNLDB and configDb.ldbNoteOnText then
			MNLDB.text = input
		end
	end
end

-----------------------------------------------------------------------
-- Global function API
--

function MobNotes_GetNote(guid)
	if not guid then return end
	return _G.MobNotesDB[guid]
end

function MobNotes_ReportTarget()
	local name = UnitName("target")
	local note = MobNotes_GetNote(getNumericUniqueID("target"))
	if not note then return end
	if GetNumRaidMembers() > 0 then
		SendChatMessage(name .. ": " .. note, "RAID")
	elseif GetNumPartyMembers() > 0 then
		SendChatMessage(name .. ": " .. note, "PARTY")
	else
		print(name .. ": " .. note)
	end
end

-----------------------------------------------------------------------
-- LDB
--

local ldb = LibStub("LibDataBroker-1.1", true)
if ldb then
	MNLDB = ldb:NewDataObject("MobNotes", {
		type = "data source",
		label = "MobNotes",
		text = "MobNotes",
		icon = "Interface\\Icons\\inv_misc_ahnqirajtrinket_03",
	})

	function MNLDB:OnClick(button)
		if button == "RightButton" then
			addNote()
		else
			MobNotes_ReportTarget()
		end
	end

	function MNLDB:OnTooltipShow()
		self:AddLine("MobNotes")
		if configDb.ldbNoteOnTip then
			local name = UnitName("target")
			if name then
				local note = MobNotes_GetNote(getNumericUniqueID("target"))
				if note then
					self:AddLine(name, 1,0,0)
					self:AddLine(note, 1,1,1, true)
					self:AddLine(" ")
				end
			end
		end
		self:AddLine("Left-Click to report note, Right-Click to add new note for current target.", 0.2, 1, 0.2, 1)
	end
end

-----------------------------------------------------------------------
-- Initialization
--

local frame = CreateFrame("Frame")
frame:SetScript("OnEvent", function(self, event)
	if event == "VARIABLES_LOADED" then
		local g = _G
		-- Set up config DB
		g.MobNotesConfig = g.MobNotesConfig or {}
		for k, v in pairs({
			ldbNoteOnTip = true,
			ldbNoteOnText = false,
			showId = false,
			hide = true, -- Minimap icon
		}) do
			if type(g.MobNotesConfig[k]) ~= type(v) then
				g.MobNotesConfig[k] = v
			end
		end
		configDb = g.MobNotesConfig

		-- Set up note DB
		if not g.MobNotesDB then
			g.MobNotesDB = defaultDatabase
			g.MobNotesDB.VERSION = DB_VERSION
		end

		local db = g.MobNotesDB

		if not db.VERSION or db.VERSION < DB_VERSION then
			for k, v in pairs(defaultDatabase) do
				if type(k) ~= "number" then
					error("MobNotes requires the default database to be populated by NPC IDs.")
				end
				if not db[k] then
					db[k] = v
				end
			end
			local num = 0
			g.OldMobNotesDB = g.OldMobNotesDB or {}
			for k, v in pairs(db) do
				if type(k) ~= "number" then
					num = num + 1
					g.OldMobNotesDB[k] = v
					db[k] = nil
				end
			end
			print("|cffff4488Mob Notes|r database updated to revision " .. tostring(DB_VERSION) .. ".")
			if num > 0 then
				print("Purged " .. num .. " entries from the local database due to internal changes in database structure. They have been stored in OldMobNotesDB, which you can find in your saved variables folder. You need to convert them manually for now, while logged out. If you search wowhead.com for a mobname, the resulting URL will end with the NPC ID.")
			end
			db.VERSION = DB_VERSION
		end

		-- Tooltip hook
		local orig = GameTooltip:GetScript("OnTooltipSetUnit")
		GameTooltip:SetScript("OnTooltipSetUnit", function(tooltip, ...)
			local name, unitid = tooltip:GetUnit()
			local id = getNumericUniqueID(unitid)
			if UnitExists(unitid) and g.MobNotesDB[id] then
				GameTooltip:AddLine(NOTE_PREFIX .. g.MobNotesDB[id], 0.5, 0.5, 0.5, 1)
			end
			if UnitExists(unitid) and configDb.showId then
				local id = getNumericUniqueID(unitid)
				if id then
					GameTooltip:AddDoubleLine("NPC ID", id)
				end
			end
			return orig(tooltip, ...)
		end)
		
		-- DBIcon
		local icon = LibStub("LibDBIcon-1.0", true)
		if MNLDB and icon then
			icon:Register("MobNotes", MNLDB, g.MobNotesConfig)
		end
	elseif event == "PLAYER_TARGET_CHANGED" then
		if not MNLDB or not configDb.ldbNoteOnText then return end
		local note = MobNotes_GetNote(getNumericUniqueID("target"))
		if note then
			MNLDB.text = note
		else
			MNLDB.text = "MobNotes"
		end
	end
end)

frame:RegisterEvent("VARIABLES_LOADED")
frame:RegisterEvent("PLAYER_TARGET_CHANGED")

_G.SlashCmdList["MOBNOTES_SHORTHAND"] = addNote
_G.SLASH_MOBNOTES_SHORTHAND1 = "/mobnotes"
_G.SLASH_MOBNOTES_SHORTHAND2 = "/mn"

